import os
import yaml

def run_command(command):
    try:
        print("[+] Running: ", command)
        return os.system(command)

    except Exception as e:
        print('[!] Error:', e)
        return 1


def read_yaml_file(file):
    try:
        with open(file, "r") as f:
            return yaml.load(f.read())
    except Exception as e:
        print('Can not read file', file)
        sys.exit(1)


def read_results(temp_file):
    return read_yaml_file(temp_file)


def write_results(results, temp_file):
    with open(temp_file, 'w') as outfile:
        yaml.dump(results, outfile, default_flow_style=False)


def print_header(text):
    print("\n------------------------------------------")
    print(text)
    print("------------------------------------------\n")


import os
import sys
from cleo import Application

from framgiaci.common import read_yaml_file

class ReportApplication(Application):

    PROJ_TYPES = ['php', 'ruby', 'android']
    TEMPLATES_DIR = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'templates')

    def __init__(self, *args, **kwargs):
        super(ReportApplication, self).__init__()

    def register_command(self, command_class):
        command = command_class()
        command.app = self
        self.add(command)

    def check_configure_file_exists(self):
        if not os.path.isfile(self.configure_file_name):
            print('.framgia-ci.yml file does not exists')
            sys.exit(1)


    def config(self, configure_file_name, temp_file_name):
        self.configure_file_name = configure_file_name
        self.temp_file_name = temp_file_name
        if os.path.isfile(configure_file_name):
            self.load_ci_reports()

    def load_ci_reports(self):
        self.ci_reports = self.parse_ci_config()
        if 'url' not in self.ci_reports:
            self.ci_reports['url'] = 'http://ci-reports.framgia.vn'

    def parse_ci_config(self):
        raw = read_yaml_file(self.configure_file_name)
        if 'from' in raw:
            template_file = os.path.join(self.TEMPLATES_DIR, "%s.yml" % raw['from'])
            base = read_yaml_file(template_file)
            defaults = {
                'comment': True,
                'ignore': False,
                'enable': True
            }
            final = {'from': raw['from']}
            final['test'] = {}
            merged_tools = [key for key in raw['test'].keys()] + [key for key in base['test'].keys()]
            merged_tools = list(set(merged_tools))
            for tool in merged_tools:
                if tool not in raw['test']:
                    final['test'][tool] = base['test'][tool]
                else:
                    final['test'][tool] = {}
                    for key, value in defaults.items():
                        final['test'][tool][key] = raw['test'][tool].get(key, base['test'][tool].get(key, defaults[key]))
                    final['test'][tool]['command'] = raw['test'][tool].get('command', base['test'][tool]['command'])

            return final
        else:
            return raw



import json
import os
import sys
import time
import pycurl

from io import BytesIO
from cleo import Command
from urllib.parse import urlencode
from framgiaci.common import read_results, print_header


class RunReportCommand(Command):
    """
    Running report command to send request to CI Report service

    report
    """

    def chmod_git_folder():
        os.system('chmod R 755 .git')

    def build_params(self):
        repo = os.environ.get('DRONE_REPO').split('/')
        return {
            'workspace': {
                'path': os.environ.get('DRONE_DIR')
            },
            'repo': {
                'owner': repo[0],
                'name': repo[1],
                'full_name': os.environ.get('DRONE_REPO')
            },
            'build': {
                'number': os.environ.get('DRONE_BUILD_NUMBER'),
                'commit': os.environ.get('DRONE_COMMIT'),
                'branch': os.environ.get('DRONE_BRANCH'),
                'pull_request_number': os.environ.get('DRONE_PULL_REQUEST')
            },
            'job': {
                'number': os.environ.get('DRONE_JOB_NUMBER')
            }
        }

    def call_api(self, url, is_post=False, params={}, headers=[]):
        buffer = BytesIO()
        c = pycurl.Curl()
        c.setopt(c.URL, url)
        c.setopt(c.WRITEDATA, buffer)
        if is_post:
            postfields = json.dumps(params)
            c.setopt(c.POSTFIELDS, postfields)
        if headers != []:
            c.setopt(c.HTTPHEADER, headers)

        c.perform()
        c.close()

        body = buffer.getvalue()
        body = body.decode('iso-8859-1')
        try:
            return json.loads(body)
        except Exception:
            return {'errorCode': 'Server Error !'}

    def create_report_queue(self, base_api_url, params):
        print('[+] Creating Report Queue ... ')
        retry_time = 10
        queue_id = None
        token = None
        for i in range(1, retry_time):
            queue_result = self.call_api(base_api_url, True, params, ['Content-Type: application/json'])
            if (queue_result and 'errorCode' in queue_result and not queue_result['errorCode']):
                queue_id = queue_result['data']['queueId']
                token = queue_result['data']['token']
                break
            else:
                print(i, 'API create report failed!')
                if ('message' in queue_result):
                    print(queue_result['message'])
            time.sleep(5)

        return queue_id, token

    def track_queue(self, queue_id, token, base_api_url):
        retry_time = 10
        print('[+] Tracking queue', queue_id)
        for i in range(1, retry_time):
            time.sleep(5)
            result = self.call_api(base_api_url + '/' + str(queue_id), False, {}, ['token:' + str(token)])
            if result and 'errorCode' in result and not result['errorCode']:
                print(result['data']['status'], result['data']['message'])
                if 'error' in result['data'] or 'success' in result['data']['status']:
                    return True if result['data']['status'] == 'success' else False
                else:
                    print(i, 'API check queue status failed!')
                    if ('message' in result):
                        print(result['message'])

        return False

    def handle(self):
        print_header('Sending Reports')
        base_api_url = self.app.ci_reports['url'] + '/api/queues'
        params = self.build_params()
        params['project_type'] = self.app.ci_reports['project_type'] if 'project_type' in self.app.ci_reports else None
        params['test_result'] = read_results(self.app.temp_file_name)
        queue_id, token = self.create_report_queue(base_api_url, params)
        if queue_id and token:
            self.track_queue(queue_id, token, base_api_url)
        else:
            sys.exit(1)
        sys.exit(0)



import sys
import os

from cleo import Command
from framgiaci.common import read_yaml_file

class CheckConfigCommand(Command):
    """
    Validate config file

    check-config
    """

    ALLOWED_BLOCKS = {
        'command': [str, list],
        'comment': bool,
        'ignore': bool,
        'enable': bool,
    }

    def handle(self):
        self.app.check_configure_file_exists()
        self.line('<info>Start checking...</info>')
        try:
            raw = read_yaml_file(self.app.configure_file_name)
            if 'from' in raw and raw['from'] not in self.app.PROJ_TYPES:
                self.line("<comment>Invalid project type '%s' !</comment>" % raw['from'])
            else:
                if 'test' not in raw:
                    self.line('<comment>No test block found !</comment>')
                else:
                    for tool, config in raw['test'].items():
                        for key in config.keys():
                            if key not in self.ALLOWED_BLOCKS.keys():
                                self.line("<comment>Block '%s' in tool '%s' is invalid !</comment>" % (key, tool))
                            else:
                                actual = config[key]
                                expected = self.ALLOWED_BLOCKS[key]
                                if isinstance(expected, list):
                                    if type(actual) not in expected:
                                        self.line("<comment>Value '%s' of block '%s' in tool '%s' is invalid !</comment>" % (actual, key, tool))
                                elif type(actual) != expected:
                                        self.line("<comment>Value '%s' of block '%s' in tool '%s' is invalid !</comment>" % (actual, key, tool))

        except Exception as e:
            self.line("<comment>Error: %s</comment>" % e)
        finally:
            self.line('<info>Done ! Your configuration is OK</info>')
